#include "StdAfx.h"
#include "RecentMessageSqlite.h"
#include "CommonType.h"
#include "RecentRecord.h"

CRecentMessageSqlite::CRecentMessageSqlite(void)
{
	m_bIsOpened = Open();
}

CRecentMessageSqlite::~CRecentMessageSqlite(void)
{
	if(m_bIsOpened)
	{
		m_bIsOpened = !Close();
	}
}

BOOL CRecentMessageSqlite::Open()
{	
	BOOL bRet = FALSE;
	CString strSQLiteFilePath = GetAppDataNtalkDir() + _T("\\Ntalkbiz.db");
	int nRet = m_sqlite3.Open(strSQLiteFilePath.GetBuffer());
	if(SQLITE_OK == nRet)
	{
		char* szSQLQueryTable = "SELECT count(*) TableCount FROM sqlite_master WHERE type='table' AND name='RecentMessage'";
		char* szSQLCreateTable = "CREATE TABLE RecentMessage(\
					 ID integer PRIMARY KEY autoincrement,\
					 OsType INT,\
					 RoomId INT,\
					 UserId VARCHAR(50),\
					 UserName VARCHAR(50),\
					 NickName VARCHAR(50),\
					 MsgId INT,\
					 MsgSeq INT,\
					 MsgGB integer,\
					 MsgStatus INT,\
					 Time VARCHAR(40),\
					 CharFormat BLOB(4096), \
					 Msg VARCHAR(6000),\
					 FileName VARCHAR(1000),\
					 FileSize UNSIGNED BIG INT,\
					 DownloadUrl VARCHAR(1000),\
					 FileFullPath VARCHAR(1000),\
					 Upload INT,\
					 JobIndex INT,\
					 FileNotifyTime VARCHAR(40),\
					 FileFailTime VARCHAR(40),\
					 CurPos INT,\
					 TotalSize INT,\
					 LastProgress INT,\
					 LastTickCout INT)";

		CSqlite3Statement stmt;
		if (m_sqlite3.CreateStatement(szSQLQueryTable,stmt) == SQLITE_OK && SQLITE_ROW == stmt.Next())
		{
			int count = stmt.GetIntValue(_T("TableCount"));
			if (count < 1)
			{
				if (SQLITE_OK == m_sqlite3.ExecuteCommand(szSQLCreateTable))
				{
					bRet = TRUE;
				}
			}
			else
			{
				bRet = TRUE;
			}
		}

		if (!bRet)
		{
			stmt.Close();
			m_sqlite3.Close();
		}
	}

	NTALK_LOG->Debug(L"[sqlite3]open sqlite3 code:%d",nRet);
	return bRet;
}

BOOL CRecentMessageSqlite::Close()
{
	int res = m_sqlite3.Close();
	NTALK_LOG->Debug(L"[sqlite3] close sqlite3 code: %d",res);

	return (SQLITE_OK == res?TRUE:FALSE);
}

BOOL CRecentMessageSqlite::AddChatItemInfo( int nRoomId, LPChatItemInfo lpChatItemInfo)
{
	USES_CONVERSION;
	BOOL bResult = FALSE;
	char sql[] = "INSERT INTO RecentMessage(OsType,RoomId,UserId,UserName,NickName,MsgId,MsgSeq,MsgGB,MsgStatus,Time,CharFormat,Msg) \
		VALUES(:OsType,:RoomId,:UserId,:UserName,:NickName,:MsgId,:MsgSeq,:MsgGB,:MsgStatus,:Time,:CharFormat,:Msg)";

	CSqlite3Statement stmt;
	if (SQLITE_OK == m_sqlite3.CreateStatement(sql,stmt))
	{
		bResult = TRUE;
		int nRet = stmt.BindInt(":OsType",lpChatItemInfo->m_nOsType);
		nRet = stmt.BindInt(":RoomId",nRoomId);
		nRet = stmt.BindText(":UserId",lpChatItemInfo->strUserID);
		nRet = stmt.BindText(":UserName",lpChatItemInfo->strUserName);
		nRet = stmt.BindText(":NickName",lpChatItemInfo->strNameTalkName);
		nRet = stmt.BindInt(":MsgId",lpChatItemInfo->nTid);
		nRet = stmt.BindInt(":MsgSeq",lpChatItemInfo->nMsgSeq);
		nRet = stmt.BindInt(":MsgGB",lpChatItemInfo->eMsgGB);
		nRet = stmt.BindInt(":MsgStatus",lpChatItemInfo->nMsgStatus);
		nRet = stmt.BindBlob(":CharFormat",(unsigned char*)&lpChatItemInfo->cf,sizeof(lpChatItemInfo->cf));
		nRet = stmt.BindText(":Msg",lpChatItemInfo->strMsg);
		nRet = stmt.BindText(":Time",CTimeToCString(lpChatItemInfo->time));
		bResult = (SQLITE_DONE == stmt.Next())?TRUE:FALSE;
	}

	return bResult;
}

BOOL CRecentMessageSqlite::UpdateMsgStatus( int nRoomId,CString strUserId,int nMsgId, int nMsgStatus)
{
	int bResult = FALSE;
	char sql[] ="UPDATE RecentMessage SET MsgStatus = :MsgStatus WHERE RoomId = :RoomId AND UserId = :UserId AND MsgId = :MsgId";
	CSqlite3Statement stmt;
	if (SQLITE_OK == m_sqlite3.CreateStatement(sql,stmt))
	{
		bResult = TRUE;
		stmt.BindInt(":RoomId",nRoomId);
		stmt.BindText(":UserId",strUserId);
		stmt.BindInt(":MsgId",nMsgId);
		stmt.BindInt(":MsgStatus",nMsgStatus);
		bResult = (SQLITE_DONE == stmt.Next())?TRUE:FALSE;
	}

	return bResult;
}

BOOL CRecentMessageSqlite::GetUserNameAndNickName(int nRoomId,int nMsgId,CString strUserId,CString &strUserName,CString &strNickName)
{
	BOOL bResult = FALSE;
	const char* szSQL = "SELECT UserName,NickName from RecentMessage WHERE RoomId = :RoomId AND MsgId=:MsgId AND UserId = :UserId";
	CSqlite3Statement stmt;
	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
	{		
		stmt.BindInt(":RoomId",nRoomId);
		stmt.BindInt(":MsgId",nMsgId);
		stmt.BindText(":UserId",strUserId);
		if (stmt.Next() == SQLITE_ROW)
		{
			strUserName = stmt.GetTextValue("UserName").GetBuffer();
			strNickName = stmt.GetTextValue("NickName").GetBuffer();
			bResult = TRUE;
		}
	}

	return bResult;
}

BOOL CRecentMessageSqlite::AddFileItemInfo( int nRoomId, TranslateFileInfoPtr pFileItemInfo )
{
	BOOL bResult = FALSE;
	char sql[] = "INSERT INTO RecentMessage(OsType,RoomId,UserId,UserName,NickName,MsgId,MsgSeq,MsgGB,MsgStatus,Time,DownloadUrl,\
				 FileName,FileSize,FileFullPath,Upload,JobIndex,FileNotifyTime,FileFailTime,CurPos,TotalSize,LastProgress,LastTickCout) VALUES(\
				 :OsType,:RoomId,:UserId,:UserName,:NickName,:MsgId,:MsgSeq,:MsgGB,:MsgStatus,:Time,\
				 :DownloadUrl,:FileName,:FileSize,:FileFullPath,:Upload,:JobIndex,:FileNotifyTime,:FileFailTime,:CurPos,:TotalSize,:LastProgress,:LastTickCout)";
	CSqlite3Statement stmt;
	if (SQLITE_OK == m_sqlite3.CreateStatement(sql,stmt))
	{
		bResult = TRUE;
		int n = stmt.BindInt(":OsType",pFileItemInfo->m_nOsType);
		 n = stmt.BindInt(":RoomId",nRoomId);
		 n = stmt.BindText(":UserId",pFileItemInfo->m_strSenderID);
		 n = stmt.BindText(":UserName",pFileItemInfo->m_strName);
		 n = stmt.BindText(":NickName",pFileItemInfo->m_strNickName);
		 n = stmt.BindInt(":MsgId",pFileItemInfo->m_TidRequestFileTranslate);
		 n = stmt.BindInt(":MsgSeq",pFileItemInfo->m_nSqliteSeq);
		 n = stmt.BindInt(":MsgGB",CHAT_MSG_FTP);
		 n = stmt.BindInt(":MsgStatus",pFileItemInfo->m_Status);
		 n = stmt.BindText(":Time",CTimeToCString(pFileItemInfo->m_tmTime));
		 n = stmt.BindText(":DownloadUrl",CString(pFileItemInfo->m_fileInfo.downloadUrl.c_str()));
		 n = stmt.BindText(":FileName",CString(pFileItemInfo->m_fileInfo.fileName.c_str()));
		 n = stmt.BindInt64(":FileSize",pFileItemInfo->m_fileInfo.fileSize);
		 n = stmt.BindText(":FileFullPath",CString(pFileItemInfo->m_fileInfo.flepath.c_str()));
		 n = stmt.BindInt(":Upload",pFileItemInfo->m_bUpload);
		 n = stmt.BindInt(":JobIndex",pFileItemInfo->m_nJobIndex);
		 n = stmt.BindText(":FileNotifyTime",CTimeToCString(pFileItemInfo->m_tmSendNotify));
		 n = stmt.BindText(":FileFailTime",CTimeToCString(pFileItemInfo->m_tmFailTime));
		 n = stmt.BindInt(":CurPos",pFileItemInfo->m_nCurPos);
		 n = stmt.BindInt(":TotalSize",pFileItemInfo->m_nTotalSize);
		 n = stmt.BindInt(":LastProgress",pFileItemInfo->m_nLastProgress);
		 n = stmt.BindInt(":LastTickCout",pFileItemInfo->m_nLastTickCout);
		bResult = (SQLITE_DONE == stmt.Next())?TRUE:FALSE;
 	}

	return bResult;
}

BOOL CRecentMessageSqlite::IsExistRecentMessage(int nRoomId,int nRecordCount)
{
	BOOL bRet = FALSE;
	char* szSQL = "SELECT Count(id) AS RecordCount FROM RecentMessage WHERE RoomId = :RoomId";

	CSqlite3Statement stmt;
	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
	{
		stmt.BindInt(":RoomId",nRoomId);
		if (SQLITE_ROW == stmt.Next())
		{
			int n = stmt.GetIntValue("RecordCount");
			if (n > nRecordCount)
			{
				bRet = TRUE;
			}
		}
	}
	return bRet;
}

int CRecentMessageSqlite::UpdateSequence(int nRoomId,CString strUserId,int nMsgId,int nSequence)
{
	int bResult = FALSE;
	char sql[] ="UPDATE RecentMessage SET msgSeq = :MsgSeq WHERE RoomId = :RoomId AND UserId = :UserId AND MsgId = :MsgId";
	CSqlite3Statement stmt;
	if (SQLITE_OK == m_sqlite3.CreateStatement(sql,stmt))
	{
		bResult = TRUE;
		stmt.BindInt(":RoomId",nRoomId);
		stmt.BindText(":UserId",strUserId);
		stmt.BindInt(":MsgId",nMsgId);
		stmt.BindInt(":MsgSeq",nSequence);
		bResult = (SQLITE_DONE == stmt.Next())?TRUE:FALSE;
	}

	return bResult;
}

int CRecentMessageSqlite::GetMsgStatus(int nRoomId,CString strUserId,int nMsgId)
{
	int nStatus = 1;
	const char* szSQL = "SELECT MsgStatus from RecentMessage WHERE RoomId = :RoomId AND UserId = :UserId AND MsgId=:MsgId";
	CSqlite3Statement stmt;
	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
	{		
		stmt.BindInt(":RoomId",nRoomId);
		stmt.BindText(":UserId",strUserId);
		stmt.BindInt(":MsgId",nMsgId);
		if (stmt.Next() == SQLITE_ROW)
		{
			nStatus = stmt.GetIntValue("MsgStatus");
		}
	}

	return nStatus;
}

int CRecentMessageSqlite::GetMaxMsgId(int nRoomId,CString strUserId)
{
	int nMaxMsgId = 1;
	const char* szSQL = "SELECT max(MsgId) AS MaxMsgId FROM RecentMessage WHERE RoomId = :RoomId AND UserId = :UserId ";
	CSqlite3Statement stmt;
	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
	{		
		stmt.BindInt(":RoomId",nRoomId);
		stmt.BindText(":UserId",strUserId);
		if (stmt.Next() == SQLITE_ROW)
		{
			nMaxMsgId = stmt.GetIntValue("MaxMsgId");
		}
	}

	return nMaxMsgId;
}

// 
// int CRecentMessageSqlite::GetMaxSequence( int nRoomId )
// {
// 	if(!m_bIsOpened)
// 	{
// 		return FALSE;
// 	}
// 
// 	int nSequence = 0;
// 	const char* szSQL = "SELECT max(MsgSeq) AS MaxMsgSeq FROM RecentMessage WHERE RoomId = :RoomId";
// 	CSqlite3Statement stmt;
// 	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
// 	{		
// 		stmt.BindInt(":RoomId",nRoomId);
// 		if (stmt.Next() == SQLITE_ROW)
// 		{
// 			nSequence = stmt.GetIntValue("MaxMsgSeq");
// 		}
// 	}
// 
// 	return nSequence;
// }



BOOL CRecentMessageSqlite::LoadRecentMessage( int nRoomId,int nBegin,int nEnd,std::vector<CRecentRecord*> &v)
{
	int nRecordCount = 0;
	int nPageCount = nEnd - nBegin;
	const char szSQL[] = "SELECT * FROM RecentMessage WHERE RoomId = :RoomIdA AND id NOT IN(\
		SELECT id FROM RecentMessage where RoomId=:RoomIdB ORDER BY id DESC LIMIT :BeginPos\
		) ORDER BY id DESC LIMIT :EndPos";

	CSqlite3Statement stmt;
	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
	{		
		stmt.BindInt(":RoomIdA",nRoomId);
		stmt.BindInt(":RoomIdB",nRoomId);
		stmt.BindInt(":BeginPos",nBegin);
		stmt.BindInt(":EndPos",nPageCount + 1);
		while (stmt.Next() == SQLITE_ROW)
		{
			if (++nRecordCount > nPageCount)
			{
				return TRUE;
			}

			ECHAT_MSG_TYPE nMsgGB = (ECHAT_MSG_TYPE)stmt.GetIntValue("MsgGB");	//message type
			if (nMsgGB != CHAT_MSG_FTP)
			{
				ChatItemInfo *pChatItemInfo =  new ChatItemInfo;
				if (pChatItemInfo)
				{
					pChatItemInfo->eMsgGB = nMsgGB;
				 	pChatItemInfo->strUserID = stmt.GetTextValue("UserId");
				  	pChatItemInfo->strNameTalkName = stmt.GetTextValue("NickName");
				 	pChatItemInfo->strUserName = stmt.GetTextValue("UserName");
				 	pChatItemInfo->strMsg = stmt.GetTextValue("Msg");
					pChatItemInfo->time =  StringToCTime(stmt.GetTextValue("Time"));
				 	pChatItemInfo->nTid = stmt.GetIntValue("MsgId");
				 	pChatItemInfo->nMsgSeq = stmt.GetIntValue("MsgSeq");
					pChatItemInfo->nMsgStatus = stmt.GetIntValue("MsgStatus");

					CHARFORMAT2 *cf = (CHARFORMAT2 *)stmt.GetBlobValue("CharFormat");
					if (cf)
					{
						pChatItemInfo->cf = *cf;
					}

					CRecentRecord *pRecentRecord = new CRecentRecord(nMsgGB,pChatItemInfo);
					if (!pRecentRecord)
					{
						delete pRecentRecord;
						pRecentRecord = NULL;

						delete pChatItemInfo;
						pChatItemInfo = NULL;
						return -1;
					}

					v.push_back(pRecentRecord);
				}
			}
			else
			{
				TranslateFileInfo *pFileInfo = (TranslateFileInfo *)new TranslateFileInfo;
				if (pFileInfo)
				{ 
					pFileInfo->m_strSenderID = stmt.GetTextValue("UserId");
					pFileInfo->m_strName = stmt.GetTextValue("UserName");
					pFileInfo->m_strNickName = stmt.GetTextValue("NickName");
					pFileInfo->m_TidRequestFileTranslate = stmt.GetIntValue("MsgId");
					pFileInfo->m_nSqliteSeq = stmt.GetIntValue("MsgSeq");
					pFileInfo->m_fileInfo.downloadUrl = stmt.GetTextValue("DownloadUrl");
					pFileInfo->m_fileInfo.fileName = stmt.GetTextValue("FileName");
					pFileInfo->m_fileInfo.fileSize = stmt.GetIntValue("FileSize");
					pFileInfo->m_fileInfo.flepath =stmt.GetTextValue("FileFullPath");
					pFileInfo->m_bUpload = stmt.GetIntValue("Upload");
					pFileInfo->m_nJobIndex = stmt.GetIntValue("JobIndex");
					pFileInfo->m_tmTime = stmt.GetTime("Time");
					pFileInfo->m_tmSendNotify = stmt.GetTime("FileNotifyTime");
					pFileInfo->m_tmFailTime = stmt.GetTime("FileFailTime");
					pFileInfo->m_nOsType = stmt.GetIntValue("OsType");
					pFileInfo->m_Status = (TransferStatus)stmt.GetIntValue("MsgStatus");
					pFileInfo->m_nCurPos = stmt.GetIntValue("CurPos");
					pFileInfo->m_nTotalSize = stmt.GetIntValue("TotalSize");
					pFileInfo->m_nLastProgress = stmt.GetIntValue("LastProgress");
					pFileInfo->m_nLastTickCout = stmt.GetIntValue("LastTickCout");

					CRecentRecord *pRecentRecord = new CRecentRecord(nMsgGB,pFileInfo);
					if (!pRecentRecord)
					{
						delete pRecentRecord;
						pRecentRecord = NULL;

						delete pFileInfo;
						pFileInfo = NULL;
						return -1;
					}
					
					v.push_back(pRecentRecord);
				}
			}
		}
	}

	return FALSE;
}



BOOL CRecentMessageSqlite::LoadTransFileInfo(int nRoomId,CString strUserId,int nMsgId,TranslateFileInfoPtr pTransFileInfo)
{
	BOOL bResult = FALSE;
	char szSQL[] = "SELECT * FROM RecentMessage WHERE RoomId = :RoomId AND UserId = :UserId AND MsgId = :MsgId AND MsgGB=:MsgGB";
	CSqlite3Statement stmt;
	if (m_sqlite3.CreateStatement(szSQL,stmt) == SQLITE_OK)
	{		 
		int n = stmt.BindInt(":RoomId",nRoomId);
		n = stmt.BindText(":UserId",strUserId);
		n = stmt.BindInt(":MsgId",nMsgId);
		n = stmt.BindInt(":MsgGB",CHAT_MSG_FTP);
		if (stmt.Next() == SQLITE_ROW)
		{
			bResult = TRUE;
			pTransFileInfo->m_strSenderID = stmt.GetTextValue("UserId");
			pTransFileInfo->m_TidRequestFileTranslate = stmt.GetIntValue("MsgId");
			pTransFileInfo->m_nSqliteSeq = stmt.GetIntValue("MsgSeq");
			pTransFileInfo->m_fileInfo.downloadUrl = stmt.GetTextValue("DownloadUrl");
			pTransFileInfo->m_fileInfo.fileName = stmt.GetTextValue("FileName");
			pTransFileInfo->m_fileInfo.fileSize = stmt.GetIntValue("FileSize");
			pTransFileInfo->m_fileInfo.flepath = stmt.GetTextValue("FileFullPath");
			pTransFileInfo->m_bUpload =  stmt.GetIntValue("Upload");
			pTransFileInfo->m_nJobIndex =  stmt.GetIntValue("JobIndex");
 			pTransFileInfo->m_tmTime = stmt.GetTime("Time");
			pTransFileInfo->m_tmSendNotify =  stmt.GetTime("FileNotifyTime");
			pTransFileInfo->m_tmFailTime =  stmt.GetTime("FileFailTime");
			pTransFileInfo->m_nOsType =   stmt.GetIntValue("OsType");
			pTransFileInfo->m_strName =   stmt.GetTextValue("UserName");
			pTransFileInfo->m_strNickName=   stmt.GetTextValue("NickName");
			pTransFileInfo->m_Status =  (TransferStatus)stmt.GetIntValue("MsgStatus");
			pTransFileInfo->m_nCurPos =  stmt.GetIntValue("CurPos");
			pTransFileInfo->m_nTotalSize =  stmt.GetIntValue("TotalSize");
			pTransFileInfo->m_nLastProgress =  stmt.GetIntValue("LastProgress");
			pTransFileInfo->m_nLastTickCout=  stmt.GetIntValue("TickCout");
		}
	}

	return bResult;
}